@@ -57,14 +57,6 @@
         <source>C&amp;hoose</source>
         <translation type="unfinished">Выбраць</translation>
     </message>
-    <message>
-        <source>Sending addresses</source>
-        <translation type="unfinished">адрасы Адпраўкі</translation>
-    </message>
-    <message>
-        <source>Receiving addresses</source>
-        <translation type="unfinished">адрасы Прымання</translation>
-    </message>
     <message>
         <source>These are your Bitcoin addresses for sending payments. Always check the amount and the receiving address before sending coins.</source>
         <translation type="unfinished">Тут знаходзяцца Біткойн-адрасы для высылання плацяжоў. Заўсёды спраўджвайце колькасць і адрас прызначэння перад здзяйсненнем транзакцыі.</translation>
@@ -464,10 +456,6 @@
         <source>Fee:</source>
         <translation type="unfinished">Камісія:</translation>
     </message>
-    <message>
-        <source>Dust:</source>
-        <translation type="unfinished">Пыл:</translation>
-    </message>
     <message>
         <source>After Fee:</source>
         <translation type="unfinished">Пасля камісіі:</translation>
@@ -528,18 +516,6 @@
         <source>Copy bytes</source>
         <translation type="unfinished">Капіяваць байты</translation>
     </message>
-    <message>
-        <source>Copy dust</source>
-        <translation type="unfinished">Капіяваць пыл</translation>
-    </message>
-    <message>
-        <source>yes</source>
-        <translation type="unfinished">так</translation>
-    </message>
-    <message>
-        <source>no</source>
-        <translation type="unfinished">не</translation>
-    </message>
     <message>
         <source>(no label)</source>
         <translation type="unfinished">непазначаны</translation>
@@ -797,10 +773,6 @@
         <source>Send to multiple recipients at once</source>
         <translation type="unfinished">Даслаць адразу некалькім атрымальнікам</translation>
     </message>
-    <message>
-        <source>Dust:</source>
-        <translation type="unfinished">Пыл:</translation>
-    </message>
     <message>
         <source>Balance:</source>
         <translation type="unfinished">Баланс:</translation>
@@ -829,10 +801,6 @@
         <source>Copy bytes</source>
         <translation type="unfinished">Капіяваць байты</translation>
     </message>
-    <message>
-        <source>Copy dust</source>
-        <translation type="unfinished">Капіяваць пыл</translation>
-    </message>
     <message>
         <source>Confirm send coins</source>
         <translation type="unfinished">Пацвердзіць дасыланне манет</translation>
@@ -974,10 +942,6 @@
         <source>Sent to</source>
         <translation type="unfinished">Даслана да</translation>
     </message>
-    <message>
-        <source>Payment to yourself</source>
-        <translation type="unfinished">Плацёж самому сабе</translation>
-    </message>
     <message>
         <source>Mined</source>
         <translation type="unfinished">Здабыта</translation>
@@ -1037,10 +1001,6 @@
         <source>Sent to</source>
         <translation type="unfinished">Даслана да</translation>
     </message>
-    <message>
-        <source>To yourself</source>
-        <translation type="unfinished">Да сябе</translation>
-    </message>
     <message>
         <source>Mined</source>
         <translation type="unfinished">Здабыта</translation>
