@@ -57,14 +57,6 @@
         <source>C&amp;hoose</source>
         <translation type="unfinished">&amp;Elekti</translation>
     </message>
-    <message>
-        <source>Sending addresses</source>
-        <translation type="unfinished">Sendaj adresoj</translation>
-    </message>
-    <message>
-        <source>Receiving addresses</source>
-        <translation type="unfinished">Ricevaj adresoj</translation>
-    </message>
     <message>
         <source>These are your Bitcoin addresses for sending payments. Always check the amount and the receiving address before sending coins.</source>
         <translation type="unfinished">Jen viaj Bitmon-adresoj por sendi pagojn. Zorge kontrolu la sumon kaj la alsendan adreson antaŭ ol sendi.</translation>
@@ -600,10 +592,6 @@ Signing is only possible with addresses of the type 'legacy'.</source>
         <source>Fee:</source>
         <translation type="unfinished">Krompago:</translation>
     </message>
-    <message>
-        <source>Dust:</source>
-        <translation type="unfinished">Polvo:</translation>
-    </message>
     <message>
         <source>After Fee:</source>
         <translation type="unfinished">Post krompago:</translation>
@@ -668,10 +656,6 @@ Signing is only possible with addresses of the type 'legacy'.</source>
         <source>Copy bytes</source>
         <translation type="unfinished">Kopii bajtojn</translation>
     </message>
-    <message>
-        <source>Copy dust</source>
-        <translation type="unfinished">Kopii polvon</translation>
-    </message>
     <message>
         <source>Copy change</source>
         <translation type="unfinished">Kopii restmonon</translation>
@@ -680,14 +664,6 @@ Signing is only possible with addresses of the type 'legacy'.</source>
         <source>(%1 locked)</source>
         <translation type="unfinished">(%1 ŝlosita)</translation>
     </message>
-    <message>
-        <source>yes</source>
-        <translation type="unfinished">jes</translation>
-    </message>
-    <message>
-        <source>no</source>
-        <translation type="unfinished">ne</translation>
-    </message>
     <message>
         <source>(no label)</source>
         <translation type="unfinished">(neniu etikedo)</translation>
@@ -1163,6 +1139,10 @@ Signing is only possible with addresses of the type 'legacy'.</source>
         <source>Close</source>
         <translation type="unfinished">Fermi</translation>
     </message>
+    <message>
+        <source>own address</source>
+        <translation type="unfinished">propra adreso</translation>
+    </message>
     <message>
         <source>Total Amount</source>
         <translation type="unfinished">Totala Sumo</translation>
@@ -1533,10 +1513,6 @@ Signing is only possible with addresses of the type 'legacy'.</source>
         <source>Clear all fields of the form.</source>
         <translation type="unfinished">Malplenigi ĉiujn kampojn de la formularo.</translation>
     </message>
-    <message>
-        <source>Dust:</source>
-        <translation type="unfinished">Polvo:</translation>
-    </message>
     <message>
         <source>Clear &amp;All</source>
         <translation type="unfinished">&amp;Forigi Ĉion</translation>
@@ -1573,10 +1549,6 @@ Signing is only possible with addresses of the type 'legacy'.</source>
         <source>Copy bytes</source>
         <translation type="unfinished">Kopii bajtojn</translation>
     </message>
-    <message>
-        <source>Copy dust</source>
-        <translation type="unfinished">Kopii polvon</translation>
-    </message>
     <message>
         <source>Copy change</source>
         <translation type="unfinished">Kopii restmonon</translation>
@@ -1955,10 +1927,6 @@ Signing is only possible with addresses of the type 'legacy'.</source>
         <source>Sent to</source>
         <translation type="unfinished">Sendita al</translation>
     </message>
-    <message>
-        <source>Payment to yourself</source>
-        <translation type="unfinished">Pago al vi mem</translation>
-    </message>
     <message>
         <source>Mined</source>
         <translation type="unfinished">Minita</translation>
@@ -2022,10 +1990,6 @@ Signing is only possible with addresses of the type 'legacy'.</source>
         <source>Sent to</source>
         <translation type="unfinished">Sendita al</translation>
     </message>
-    <message>
-        <source>To yourself</source>
-        <translation type="unfinished">Al vi mem</translation>
-    </message>
     <message>
         <source>Mined</source>
         <translation type="unfinished">Minita</translation>
