@@ -632,6 +632,13 @@
         <translation type="unfinished">Fossum pendendum quod nondum maturum est</translation>
     </message>
     </context>
+<context>
+    <name>PSBTOperationsDialog</name>
+    <message>
+        <source>own address</source>
+        <translation type="unfinished">inscriptio propria</translation>
+    </message>
+    </context>
 <context>
     <name>PaymentServer</name>
     <message>
@@ -1153,10 +1160,6 @@
         <source>Sent to</source>
         <translation type="unfinished">Missum ad</translation>
     </message>
-    <message>
-        <source>Payment to yourself</source>
-        <translation type="unfinished">Pensitatio ad te ipsum</translation>
-    </message>
     <message>
         <source>Mined</source>
         <translation type="unfinished">Fossa</translation>
@@ -1216,10 +1219,6 @@
         <source>Sent to</source>
         <translation type="unfinished">Missum ad</translation>
     </message>
-    <message>
-        <source>To yourself</source>
-        <translation type="unfinished">Ad te ipsum</translation>
-    </message>
     <message>
         <source>Mined</source>
         <translation type="unfinished">Fossa</translation>
