@@ -59,14 +59,6 @@ Alihkan fail data ke dalam tab semasa</translation>
         <source>C&amp;hoose</source>
         <translation type="unfinished">&amp;Pilih</translation>
     </message>
-    <message>
-        <source>Sending addresses</source>
-        <translation type="unfinished">alamat-alamat penghantaran</translation>
-    </message>
-    <message>
-        <source>Receiving addresses</source>
-        <translation type="unfinished">alamat-alamat penerimaan</translation>
-    </message>
     <message>
         <source>These are your Bitcoin addresses for sending payments. Always check the amount and the receiving address before sending coins.</source>
         <translation type="unfinished">Ini adalah alamat Bitcoin anda untuk pembayaran. Periksa jumlah dan alamat penerima sebelum membuat penghantaran koin sentiasa.</translation>
