@@ -49,14 +49,6 @@
         <source>C&amp;hoose</source>
         <translation type="unfinished">P&amp;ilinan</translation>
     </message>
-    <message>
-        <source>Sending addresses</source>
-        <translation type="unfinished">Address king pamag-Send</translation>
-    </message>
-    <message>
-        <source>Receiving addresses</source>
-        <translation type="unfinished">Address king pamag-Tanggap</translation>
-    </message>
     <message>
         <source>These are your Bitcoin addresses for sending payments. Always check the amount and the receiving address before sending coins.</source>
         <translation type="unfinished">Reni reng kekang Bitcoin address king pamagpadalang kabayaran. Lawan mulang masalese reng alaga ampo ing address na ning tumanggap bayu ka magpadalang barya.</translation>
@@ -541,6 +533,13 @@
         <translation type="unfinished">Ing kekang kasalungsungan kabuuang balanse</translation>
     </message>
     </context>
+<context>
+    <name>PSBTOperationsDialog</name>
+    <message>
+        <source>own address</source>
+        <translation type="unfinished">sariling address</translation>
+    </message>
+    </context>
 <context>
     <name>PeerTableModel</name>
     <message>
@@ -968,10 +967,6 @@
         <source>Sent to</source>
         <translation type="unfinished">Mipadala kang</translation>
     </message>
-    <message>
-        <source>Payment to yourself</source>
-        <translation type="unfinished">Kabayaran keka</translation>
-    </message>
     <message>
         <source>Mined</source>
         <translation type="unfinished">Me-mina</translation>
@@ -1031,10 +1026,6 @@
         <source>Sent to</source>
         <translation type="unfinished">Mipadala kang</translation>
     </message>
-    <message>
-        <source>To yourself</source>
-        <translation type="unfinished">Keng sarili mu</translation>
-    </message>
     <message>
         <source>Mined</source>
         <translation type="unfinished">Me-mina</translation>
