@@ -17,7 +17,8 @@ FUZZ_TARGET(fees)
 {
     FuzzedDataProvider fuzzed_data_provider(buffer.data(), buffer.size());
     const CFeeRate minimal_incremental_fee{ConsumeMoney(fuzzed_data_provider)};
-    FeeFilterRounder fee_filter_rounder{minimal_incremental_fee};
+    FastRandomContext rng{/*fDeterministic=*/true};
+    FeeFilterRounder fee_filter_rounder{minimal_incremental_fee, rng};
     LIMITED_WHILE(fuzzed_data_provider.ConsumeBool(), 10000) {
         const CAmount current_minimum_fee = ConsumeMoney(fuzzed_data_provider);
         const CAmount rounded_fee = fee_filter_rounder.round(current_minimum_fee);
