@@ -1,4 +1,4 @@
-// Copyright (c) 2019-2021 The Bitcoin Core developers
+// Copyright (c) 2019-2022 The Bitcoin Core developers
 // Distributed under the MIT software license, see the accompanying
 // file COPYING or http://www.opensource.org/licenses/mit-license.php.
 
@@ -6,12 +6,15 @@
 
 #include <clientversion.h>
 #include <crypto/common.h>
-#include <fs.h>
 #include <logging.h>
+#include <serialize.h>
 #include <streams.h>
+#include <util/fs.h>
 
+#include <algorithm>
 #include <cassert>
-#include <map>
+#include <cstdio>
+#include <utility>
 #include <vector>
 
 namespace {
@@ -195,7 +198,7 @@ std::vector<bool> DecodeAsmap(fs::path path)
 {
     std::vector<bool> bits;
     FILE *filestr = fsbridge::fopen(path, "rb");
-    CAutoFile file(filestr, SER_DISK, CLIENT_VERSION);
+    AutoFile file{filestr};
     if (file.IsNull()) {
         LogPrintf("Failed to open asmap file from disk\n");
         return bits;
