@@ -28,9 +28,6 @@ class VersionBitsWarningTest(BitcoinTestFramework):
     def set_test_params(self):
         self.setup_clean_chain = True
         self.num_nodes = 1
-        # The experimental syscall sandbox feature (-sandbox) is not compatible with -alertnotify
-        # (which invokes execve).
-        self.disable_syscall_sandbox = True
 
     def setup_network(self):
         self.alert_filename = os.path.join(self.options.tmpdir, "alert.txt")
