@@ -75,7 +75,7 @@ class Variant(collections.namedtuple("Variant", "call data address_type rescan p
                 request.update({"redeemscript": self.address['embedded']['scriptPubKey']})
             response = self.node.importmulti(
                 requests=[request],
-                options={"rescan": self.rescan in (Rescan.yes, Rescan.late_timestamp)},
+                rescan=self.rescan in (Rescan.yes, Rescan.late_timestamp),
             )
             assert_equal(response, [{"success": True}])
 
